@@ -17,8 +17,8 @@ except ImportError:
     log.warning('Scipy import failed!')
 
 
-def oriented_bounds_2D(points):
-    '''
+def oriented_bounds_2D(points, qhull_options='QbB'):
+    """
     Find an oriented bounding box for a set of 2D points.
 
     Parameters
@@ -27,24 +27,30 @@ def oriented_bounds_2D(points):
 
     Returns
     ----------
-    transform: (3,3) float, homogenous 2D transformation matrix to move the input set of
-               points so that the axis aligned bounding box is CENTERED AT THE ORIGIN
-    rectangle: (2,) float, size of extents once input points are transformed by transform
-    '''
+    transform: (3,3) float, homogenous 2D transformation matrix to move the
+                input points so that the axis aligned bounding box
+                is CENTERED AT THE ORIGIN
+    rectangle: (2,) float, size of extents once input points are transformed
+                by transform
+    """
     # make sure input is a numpy array
     points = np.asanyarray(points)
     # create a convex hull object of our points
     # 'QbB' is a qhull option which has it scale the input to unit box
     # to avoid precision issues with very large/small meshes
-    convex = spatial.ConvexHull(points, qhull_options='QbB')
+    convex = spatial.ConvexHull(points,
+                                qhull_options=qhull_options)
 
     # (n,2,3) line segments
     hull_edges = convex.points[convex.simplices]
     # (n,2) points on the convex hull
     hull_points = convex.points[convex.vertices]
 
-    # unitize the direction of the edges of the hull polygon
-    edge_vectors = util.unitize(np.diff(hull_edges, axis=1).reshape((-1, 2)))
+    # direction of the edges of the hull polygon
+    edge_vectors = np.diff(hull_edges, axis=1).reshape((-1, 2))
+
+    # unitize vectors
+    edge_vectors /= np.linalg.norm(edge_vectors, axis=1).reshape((-1, 1))
     # create a set of perpendicular vectors
     perp_vectors = np.fliplr(edge_vectors) * [-1.0, 1.0]
 
@@ -63,25 +69,37 @@ def oriented_bounds_2D(points):
                               x.max(axis=1),
                               y.max(axis=1)))
 
-    # calculate the extents and area that a box drawn around each edge_vector
-    extents = np.diff(bounds.reshape((-1, 2, 2)), axis=1).reshape((-1, 2))
+    # calculate the extents and area for each edge vector pair
+    extents = np.diff(bounds.reshape((-1, 2, 2)),
+                      axis=1).reshape((-1, 2))
     area = np.product(extents, axis=1)
     area_min = area.argmin()
 
     #(2,) float of smallest rectangle size
     rectangle = extents[area_min]
 
-    # find the (3,3) homogenous transformation which moves the input points
-    # to have a bounding box centered at the origin
-    offset = -bounds[area_min][0:2] - (rectangle * .5)
+    # find the (3,3) homogenous transformation which moves the input
+    # points to have a bounding box centered at the origin
+    offset = -bounds[area_min][:2] - (rectangle * .5)
     theta = np.arctan2(*edge_vectors[area_min][::-1])
-    transform = transformations.planar_matrix(offset, theta)
+    transform = transformations.planar_matrix(offset,
+                                              theta)
+
+    # we would like to consistently return an OBB with
+    # the largest dimension along the X axis
+    if np.less(*rectangle):
+        # a 90 degree rotation
+        flip = transformations.planar_matrix(theta=np.pi / 2)
+        # apply the rotation
+        transform = np.dot(flip, transform)
+        # switch X and Y in the OBB extents
+        rectangle = np.roll(rectangle, 1)
 
     return transform, rectangle
 
 
 def oriented_bounds(obj, angle_digits=1):
-    '''
+    """
     Find the oriented bounding box for a Trimesh
 
     Parameters
@@ -99,7 +117,7 @@ def oriented_bounds(obj, angle_digits=1):
     to_origin: (4,4) float, transformation matrix which will move the center of the
                bounding box of the input mesh to the origin.
     extents: (3,) float, the extents of the mesh once transformed with to_origin
-    '''
+    """
 
     # extract a set of convex hull vertices and normals from the input
     # we bother to do this to avoid recomputing the full convex hull if
@@ -144,7 +162,10 @@ def oriented_bounds(obj, angle_digits=1):
     for spherical in spherical_coords[spherical_unique]:
         # a matrix which will rotate each hull normal to [0,0,1]
         to_2D = np.linalg.inv(transformations.spherical_matrix(*spherical))
-        projected = transformations.transform_points(vertices, to_2D)
+        # apply the transform here
+        projected = np.dot(to_2D,
+                           np.column_stack((vertices,
+                                            np.ones(len(vertices)))).T).T[:, :3]
 
         height = projected[:, 2].ptp()
         rotation_2D, box = oriented_bounds_2D(projected[:, 0:2])
@@ -161,7 +182,8 @@ def oriented_bounds(obj, angle_digits=1):
 
     # transform points using our matrix to find the translation for the
     # transform
-    transformed = transformations.transform_points(vertices, to_origin)
+    transformed = transformations.transform_points(vertices,
+                                                   to_origin)
     box_center = (transformed.min(axis=0) + transformed.ptp(axis=0) * .5)
     to_origin[0:3, 3] = -box_center
 
@@ -173,7 +195,7 @@ def oriented_bounds(obj, angle_digits=1):
 
 
 def minimum_cylinder(obj, sample_count=10, angle_tol=.001):
-    '''
+    """
     Find the approximate minimum volume cylinder which contains a mesh or
     list of points.
 
@@ -198,10 +220,10 @@ def minimum_cylinder(obj, sample_count=10, angle_tol=.001):
                 'height'    : float, height of cylinder
                 'transform' : (4,4) float, transform from the origin
                                to centered cylinder
-    '''
+    """
 
     def volume_from_angles(spherical, return_data=False):
-        '''
+        """
         Takes spherical coordinates and calculates the volume of a cylinder
         along that vector
 
@@ -218,7 +240,7 @@ def minimum_cylinder(obj, sample_count=10, angle_tol=.001):
             height (float)
         else:
             volume (float)
-        '''
+        """
         to_2D = transformations.spherical_matrix(*spherical, axes='rxyz')
         projected = transformations.transform_points(hull, matrix=to_2D)
         height = projected[:, 2].ptp()
@@ -246,8 +268,9 @@ def minimum_cylinder(obj, sample_count=10, angle_tol=.001):
     samples = util.grid_linspace([[0, 0], [np.pi, np.pi]], sample_count)
     # add the principal inertia vectors if we have a mesh
     if hasattr(obj, 'principal_inertia_vectors'):
-        samples = np.vstack((samples,
-                             util.vector_to_spherical(obj.principal_inertia_vectors)))
+        samples = np.vstack(
+            (samples, util.vector_to_spherical(
+                obj.principal_inertia_vectors)))
     tic = [time.time()]
     # the projected volume at each sample
     volumes = np.array([volume_from_angles(i) for i in samples])
@@ -279,7 +302,7 @@ def minimum_cylinder(obj, sample_count=10, angle_tol=.001):
 
 
 def corners(bounds):
-    '''
+    """
     Given a pair of axis aligned bounds, return all
     8 corners of the bounding box.
 
@@ -290,7 +313,7 @@ def corners(bounds):
     Returns
     ----------
     corners: (8,3) float, corner vertices of the cube
-    '''
+    """
 
     bounds = np.asanyarray(bounds, dtype=np.float64)
 
@@ -314,7 +337,7 @@ def corners(bounds):
 
 
 def contains(bounds, points):
-    '''
+    """
     Do an axis aligned bounding box check on a list of points.
 
     Parameters
@@ -325,7 +348,7 @@ def contains(bounds, points):
     Returns
     -----------
     points_inside: (n,) bool, True if points are inside the AABB
-    '''
+    """
     bounds = np.asanyarray(bounds, dtype=np.float64)
     points = np.asanyarray(points)
 
