@@ -1,3 +1,14 @@
+"""
+graph.py
+-------------
+
+Deal with graph operations. Primarily deal with graphs in (n,2)
+edge list form, and abstract the backend graph library being used.
+
+Currently uses networkx, scipy.sparse.csgraph, or graph_tool
+backends.
+"""
+
 import numpy as np
 import networkx as nx
 import collections
@@ -14,7 +25,6 @@ try:
     _has_gt = True
 except ImportError:
     _has_gt = False
-    log.warning('graph-tool unavailable, some operations will be much slower')
 
 try:
     from scipy.sparse import csgraph, coo_matrix
@@ -25,7 +35,7 @@ except ImportError:
 def face_adjacency(faces=None,
                    mesh=None,
                    return_edges=False):
-    '''
+    """
     Returns an (n,2) list of face indices.
     Each pair of faces in the list shares an edge, making them adjacent.
 
@@ -51,7 +61,7 @@ def face_adjacency(faces=None,
     graph = nx.Graph()
     graph.add_edges_from(mesh.face_adjacency)
     groups = nx.connected_components(graph_connected)
-    '''
+    """
 
     if mesh is None:
         # first generate the list of edges for the current faces
@@ -87,7 +97,7 @@ def face_adjacency(faces=None,
 
 
 def face_adjacency_unshared(mesh):
-    '''
+    """
     Return the vertex index of the two vertices not in the shared
     edge between two adjacent faces
 
@@ -98,7 +108,7 @@ def face_adjacency_unshared(mesh):
     Returns
     -----------
     vid_unshared: (len(mesh.face_adjacency), 2) int, indexes of mesh.vertices
-    '''
+    """
 
     # the non- shared vertex index is the same shape as face_adjacnecy
     # just holding vertex indices rather than face indices
@@ -115,7 +125,7 @@ def face_adjacency_unshared(mesh):
 
 
 def face_adjacency_radius(mesh):
-    '''
+    """
     Compute an approximate radius between adjacent faces.
 
     Parameters
@@ -128,7 +138,7 @@ def face_adjacency_radius(mesh):
            Parallel faces will have a value of np.inf
     span:  (n,) float, perpendicular projection distance of two
            unshared vertices onto the shared edge
-    '''
+    """
 
     # solve for the radius of the adjacent faces
     #         distance
@@ -151,9 +161,10 @@ def face_adjacency_radius(mesh):
                                      axis=1).reshape((-1, 3)))
 
     # the vector of the perpendicular projection to the shared edge
-    perp = np.subtract(vectors,
-                       (util.diagonal_dot(vectors,
-                                          edges_vec).reshape((-1, 1)) * edges_vec))
+    perp = np.subtract(
+        vectors, (util.diagonal_dot(
+            vectors, edges_vec).reshape(
+            (-1, 1)) * edges_vec))
     # the length of the perpendicular projection
     span = np.linalg.norm(perp, axis=1)
 
@@ -165,7 +176,7 @@ def face_adjacency_radius(mesh):
 
 
 def vertex_adjacency_graph(mesh):
-    '''
+    """
     Returns a networkx graph representing the vertices and their connections
     in the mesh.
 
@@ -188,7 +199,7 @@ def vertex_adjacency_graph(mesh):
     graph = mesh.vertex_adjacency_graph
     graph.neighbors(0)
     > [1,3,4]
-    '''
+    """
 
     g = nx.Graph()
     g.add_edges_from(mesh.edges_unique)
@@ -196,7 +207,7 @@ def vertex_adjacency_graph(mesh):
 
 
 def shared_edges(faces_a, faces_b):
-    '''
+    """
     Given two sets of faces, find the edges which are in both sets.
 
     Parameters
@@ -207,19 +218,19 @@ def shared_edges(faces_a, faces_b):
     Returns
     ---------
     shared: (p, 2) int, set of edges
-    '''
+    """
     e_a = np.sort(faces_to_edges(faces_a), axis=1)
     e_b = np.sort(faces_to_edges(faces_b), axis=1)
-    shared = grouping.boolean_rows(e_a, e_b, operation=set.intersection)
+    shared = grouping.boolean_rows(e_a, e_b, operation=np.intersect1d)
     return shared
 
 
 def connected_edges(G, nodes):
-    '''
+    """
     Given graph G and list of nodes, return the list of edges that
     are connected to nodes
 
-    '''
+    """
     nodes_in_G = collections.deque()
     for node in nodes:
         if not G.has_node(node):
@@ -230,7 +241,7 @@ def connected_edges(G, nodes):
 
 
 def facets(mesh, engine=None):
-    '''
+    """
     Find the list of parallel adjacent faces.
 
     Parameters
@@ -240,8 +251,9 @@ def facets(mesh, engine=None):
 
     Returns
     ---------
-    facets: list of groups of face indexes (mesh.faces) of parallel adjacent faces.
-    '''
+    facets: list of groups of face indexes (mesh.faces) of parallel
+                  adjacent faces.
+    """
     # what is the radius of a circle that passes through the perpendicular
     # projection of the vector between the two non- shared vertices
     # onto the shared edge, with the face normal from the two adjacent faces
@@ -254,7 +266,12 @@ def facets(mesh, engine=None):
     # a common failure mode is two faces that are very narrow with a slight
     # angle between them, so here we divide by the perpendicular span
     # to penalize very narrow faces, and then square it just for fun
-    parallel = (radii / span) ** 2 > tol.facet_threshold
+    parallel = np.ones(len(radii), dtype=np.bool)
+    # if span is zero we know faces are small/parallel
+    nonzero = np.abs(span) > tol.zero
+    # faces with a radii/span ratio larger than a threshold pass
+    parallel[nonzero] = (radii[nonzero] /
+                         span[nonzero]) ** 2 > tol.facet_threshold
 
     # run connected components on the parallel faces to group them
     components = connected_components(mesh.face_adjacency[parallel],
@@ -268,7 +285,7 @@ def split(mesh,
           only_watertight=True,
           adjacency=None,
           engine=None):
-    '''
+    """
     Split a mesh into multiple meshes from face connectivity.
 
     If only_watertight is true, it will only return watertight meshes
@@ -285,7 +302,7 @@ def split(mesh,
     Returns
     ----------
     meshes: list of Trimesh objects
-    '''
+    """
 
     if adjacency is None:
         adjacency = mesh.face_adjacency
@@ -309,7 +326,7 @@ def connected_components(edges,
                          min_len=1,
                          nodes=None,
                          engine=None):
-    '''
+    """
     Find groups of connected nodes from an edge list.
 
     Parameters
@@ -324,11 +341,11 @@ def connected_components(edges,
     Returns
     -----------
     components: (n,) sequence of lists, nodes which are connected
-    '''
+    """
     def components_networkx():
-        '''
+        """
         Find connected components using networkx
-        '''
+        """
         graph = nx.from_edgelist(edges)
         # make sure every face has a node, so single triangles
         # aren't discarded (as they aren't adjacent to anything)
@@ -341,9 +358,9 @@ def connected_components(edges,
         return components
 
     def components_graphtool():
-        '''
+        """
         Find connected components using graphtool
-        '''
+        """
         g = GTGraph()
         # make sure all the nodes are in the graph
         g.add_vertex(node_count)
@@ -365,9 +382,9 @@ def connected_components(edges,
         return components
 
     def components_csgraph():
-        '''
+        """
         Find connected components using scipy.sparse.csgraph
-        '''
+        """
         # label each node
         labels = connected_component_labels(edges,
                                             node_count=node_count)
@@ -438,7 +455,7 @@ def connected_components(edges,
 
 
 def connected_component_labels(edges, node_count=None):
-    '''
+    """
     Label graph nodes from an edge list, using scipy.sparse.csgraph
 
     Parameters
@@ -449,7 +466,7 @@ def connected_component_labels(edges, node_count=None):
     Returns
     ---------
     labels: (node_count,) int, component labels for each node
-    '''
+    """
     matrix = edges_to_coo(edges, node_count)
     body_count, labels = csgraph.connected_components(matrix,
                                                       directed=False)
@@ -460,7 +477,7 @@ def connected_component_labels(edges, node_count=None):
 
 
 def dfs_traversals(edges):
-    '''
+    """
     Given an edge list, generate a sequence of ordered
     depth first search traversals, using scipy.csgraph routines.
 
@@ -472,7 +489,7 @@ def dfs_traversals(edges):
     -----------
     traversals: (m,) sequence of (p,) int,
                 ordered DFS traversals of the graph.
-    '''
+    """
     edges = np.asanyarray(edges, dtype=np.int64)
     if not util.is_shape(edges, (-1, 2)):
         raise ValueError('edges are not (n,2)!')
@@ -510,7 +527,7 @@ def dfs_traversals(edges):
 
 
 def edges_to_coo(edges, count=None):
-    '''
+    """
     Given an edge list, return a boolean scipy.sparse.coo_matrix
     representing the edges in matrix form.
 
@@ -523,7 +540,7 @@ def edges_to_coo(edges, count=None):
     Returns
     ------------
     matrix: (count, count) bool, scipy.sparse.coo_matrix
-    '''
+    """
     edges = np.asanyarray(edges, dtype=np.int64)
     if not (len(edges) == 0 or
             util.is_shape(edges, (-1, 2))):
@@ -543,7 +560,7 @@ def edges_to_coo(edges, count=None):
 
 
 def smoothed(mesh, angle):
-    '''
+    """
     Return a non- watertight version of the mesh which will
     render nicely with smooth shading.
 
@@ -556,7 +573,7 @@ def smoothed(mesh, angle):
     Returns
     ---------
     smooth: Trimesh object
-    '''
+    """
     if len(mesh.face_adjacency) == 0:
         return mesh
     angle_ok = mesh.face_adjacency_angles <= angle
@@ -571,27 +588,35 @@ def smoothed(mesh, angle):
 
 
 def is_watertight(edges, edges_sorted=None):
-    '''
+    """
     Parameters
     ---------
-    edges: (n,2) int, set of vertex indices
+    edges:        (n, 2) int, set of vertex indices
+    edges_sorted: (n, 2) int, vertex indices sorted on axis 1
 
     Returns
     ---------
-    watertight: boolean, whether every edge is contained by two faces
-    '''
+    watertight: boolean, whether every edge is shared by an even
+                number of faces
+    winding:    boolean, whether every shared edge is reversed
+    """
+    # passing edges_sorted is a speedup only
     if edges_sorted is None:
         edges_sorted = np.sort(edges, axis=1)
+
+    # group sorted edges
     groups = grouping.group_rows(edges_sorted, require_count=2)
     watertight = (len(groups) * 2) == len(edges)
 
+    # are opposing edges reversed
     opposing = edges[groups].reshape((-1, 4))[:, 1:3].T
-    reversed = np.equal(*opposing).all()
-    return watertight, reversed
+    winding = np.equal(*opposing).all()
+
+    return bool(watertight), bool(winding)
 
 
 def graph_to_svg(graph):
-    '''
+    """
     Turn a networkx graph into an SVG string, using graphviz dot.
 
     Parameters
@@ -601,7 +626,7 @@ def graph_to_svg(graph):
     Returns
     ---------
     svg: string, pictoral layout in SVG format
-    '''
+    """
 
     import tempfile
     import subprocess
@@ -609,3 +634,93 @@ def graph_to_svg(graph):
         nx.drawing.nx_agraph.write_dot(graph, dot_file.name)
         svg = subprocess.check_output(['dot', dot_file.name, '-Tsvg'])
     return svg
+
+
+def multigraph_paths(G, source, cutoff=None):
+    """
+    For a networkx MultiDiGraph, find all paths from a source node
+    to leaf nodes. This function returns edge instance numbers
+    in addition to nodes, unlike networkx.all_simple_paths.
+
+    Parameters
+    ---------------
+    G: networkx.MultiDiGraph
+    source: str, node to start off
+    cutoff: int, number of nodes to visit
+                 if None, will
+
+    Returns
+    ----------
+    traversals: (n,) list of [(node, edge instance index), ] paths
+    """
+    if cutoff is None:
+        cutoff = (len(G.edges()) * len(G.nodes())) + 1
+
+    # the path starts at the node specified
+    current = [(source, 0)]
+    # traversals we need to go back and do
+    queue = []
+    # completed paths
+    traversals = []
+
+    for i in range(cutoff):
+        # paths are stored as (node, instance) so
+        # get the node of the last place visited
+        current_node = current[-1][0]
+        # get all the children of the current node
+        child = G[current_node]
+
+        if len(child) == 0:
+            # we have no children, so we are at the end of this path
+            # save the path as a completed traversal
+            traversals.append(current)
+            # if there is nothing on the queue, we are done
+            if len(queue) == 0:
+                break
+            # otherwise continue traversing with the next path
+            # on the queue
+            current = queue.pop()
+        else:
+            # oh no, we have multiple edges from current -> child
+            start = True
+            # iterate through child nodes and edge instances
+            for node in child.keys():
+                for instance in child[node].keys():
+                    if start:
+                        # if this is the first edge, keep it on the
+                        # current traversal and save the others for later
+                        current.append((node, instance))
+                        start = False
+                    else:
+                        # this child has multiple instances
+                        # so we will need to traverse them multiple times
+                        # we appended a node to current, so only take the
+                        # first n-1 visits
+                        queue.append(current[:-1] + [(node, instance)])
+    return traversals
+
+
+def multigraph_collect(G, traversal, attrib=None):
+    """
+    Given a MultiDiGraph traversal, collect attributes along that
+    path.
+
+    Parameters
+    -------------
+    G:          networkx.MultiDiGraph
+    traversal:  (n) list of (node, instance) tuples
+    attrib:     dict key, name to collect. If None, will return all
+
+    Returns
+    -------------
+    collected: (len(traversal) - 1) list of attributes
+    """
+
+    collected = []
+    for u, v in util.pairwise(traversal):
+        attribs = G[u[0]][v[0]][v[1]]
+        if attrib is None:
+            collected.append(attribs)
+        else:
+            collected.append(attribs[attrib])
+    return collected
