@@ -1,12 +1,18 @@
+"""
+grouping.py
+-------------
+
+Functions for grouping values and rows.
+"""
+
 import numpy as np
-import networkx as nx
-
-from . import util
-
-from .constants import log, tol
 
 from collections import deque
 
+
+from . import util
+from .constants import log, tol
+
 try:
     from scipy.spatial import cKDTree as KDTree
 except ImportError:
@@ -14,33 +20,16 @@ except ImportError:
 
 
 def merge_vertices_hash(mesh):
-    '''
+    """
     Removes duplicate vertices, based on integer hashes.
     This is roughly 20x faster than querying a KD tree in a loop
-    '''
+    """
     unique, inverse = unique_rows(mesh.vertices)
     mesh.update_vertices(unique, inverse)
 
 
-def replace_references(data, reference_dict):
-    '''
-    Replace elements in an array as per a dictionary of replacement values.
-
-    Parameters
-    ----------
-    data:           numpy array
-    reference_dict: dictionary of replacement value mapping, eg: {2:1, 3:1, 4:5}
-    '''
-    shape = np.shape(data)
-    view = np.array(data).view().reshape((-1))
-    for i, value in enumerate(view):
-        if value in reference_dict:
-            view[i] = reference_dict[value]
-    return view.reshape(shape)
-
-
 def group(values, min_len=0, max_len=np.inf):
-    '''
+    """
     Return the indices of values that are identical
 
     Parameters
@@ -55,7 +44,7 @@ def group(values, min_len=0, max_len=np.inf):
     ----------
     groups: sequence of indices to form groups
             IE [0,1,0,1] returns [[0,2], [1,3]]
-    '''
+    """
     original = np.asanyarray(values)
 
     # save the sorted order and then apply it
@@ -74,7 +63,7 @@ def group(values, min_len=0, max_len=np.inf):
         nondupe = values[1:] != values[:-1]
 
     dupe_idx = np.append(0, np.nonzero(nondupe)[0] + 1)
-    dupe_len = np.diff(np.hstack((dupe_idx, len(values))))
+    dupe_len = np.diff(np.concatenate((dupe_idx, [len(values)])))
     dupe_ok = np.logical_and(np.greater_equal(dupe_len, min_len),
                              np.less_equal(dupe_len, max_len))
     groups = [order[i:(i + j)]
@@ -86,9 +75,9 @@ def group(values, min_len=0, max_len=np.inf):
 
 
 def hashable_rows(data, digits=None):
-    '''
-    We turn our array into integers, based on the precision
-    given by digits, and then put them in a hashable format.
+    """
+    We turn our array into integers based on the precision
+    given by digits and then put them in a hashable format.
 
     Parameters
     ---------
@@ -100,21 +89,67 @@ def hashable_rows(data, digits=None):
     ---------
     hashable:  (n) length array of custom data which can be sorted
                 or used as hash keys
-    '''
-    as_int = float_to_int(data, digits)
+    """
+    # if there is no data return immediatly
+    if len(data) == 0:
+        return np.array([])
+
+    # get array as integer to precision we care about
+    as_int = float_to_int(data, digits=digits)
+
+    # if it is flat integers already, return
+    if len(as_int.shape) == 1:
+        return as_int
+
+    # if array is 2D and smallish, we can try bitbanging
+    # this is signifigantly faster than the custom dtype
+    if len(as_int.shape) == 2 and as_int.shape[1] <= 4:
+        # time for some righteous bitbanging
+        # can we pack the whole row into a single 64 bit integer
+        precision = int(np.floor(64 / as_int.shape[1]))
+        # if the max value is less than precision we can do this
+        if np.abs(as_int).max() < 2**(precision - 1):
+            # the resulting package
+            hashable = np.zeros(len(as_int), dtype=np.int64)
+            # loop through each column and bitwise xor to combine
+            # make sure as_int is int64 otherwise bit offset won't work
+            for offset, column in enumerate(as_int.astype(np.int64).T):
+                # will modify hashable in place
+                np.bitwise_xor(hashable,
+                               column << (offset * precision),
+                               out=hashable)
+            return hashable
+
+    # reshape array into magical data type that is weird but hashable
     dtype = np.dtype((np.void, as_int.dtype.itemsize * as_int.shape[1]))
+    # make sure result is contiguous and flat
     hashable = np.ascontiguousarray(as_int).view(dtype).reshape(-1)
     return hashable
 
 
-def float_to_int(data, digits=None, dtype_out=np.int32):
-    '''
-    Given a numpy array of data represent it as integers.
-    '''
+def float_to_int(data, digits=None, dtype=np.int32):
+    """
+    Given a numpy array of float/bool/int, return as integers.
+
+    Parameters
+    -------------
+    data:   (n, d) float, int, or bool data
+    digits: float/int precision for float conversion
+    dtype:  numpy dtype for result
+
+    Returns
+    -------------
+    as_int: data, as integers
+    """
+    # convert to any numpy array
     data = np.asanyarray(data)
 
-    if data.size == 0:
-        return data.astype(dtype_out)
+    # if data is already an integer or boolean we're done
+    # if the data is empty we are also done
+    if data.dtype.kind in 'ib' or data.size == 0:
+        return data.astype(dtype)
+
+    # populate digits from kwargs
     if digits is None:
         digits = util.decimal_to_digits(tol.merge)
     elif isinstance(digits, float) or isinstance(digits, np.float):
@@ -123,19 +158,18 @@ def float_to_int(data, digits=None, dtype_out=np.int32):
         log.warn('Digits were passed as %s!', digits.__class__.__name__)
         raise ValueError('Digits must be None, int, or float!')
 
-    # if data is already an integer or boolean, we're done
-    if data.dtype.kind in 'ib':
-        as_int = data.astype(dtype_out)
-    else:
-        data_max = np.abs(data).max() * 10**digits
-        dtype_out = [np.int32, np.int64][int(data_max > 2**31)]
-        as_int = (np.round(data, digits) * (10**digits)).astype(dtype_out)
+    # data is float so convert to large integers
+    data_max = np.abs(data).max() * 10**digits
+    # ignore passed dtype if we have something large
+    dtype = [np.int32, np.int64][int(data_max > 2**31)]
+    # round, multiply by large number, and convert
+    as_int = (np.round(data, digits) * (10**digits)).astype(dtype)
 
     return as_int
 
 
 def unique_ordered(data):
-    '''
+    """
     Returns the same as np.unique, but ordered as per the
     first occurance of the unique value in data.
 
@@ -148,7 +182,7 @@ def unique_ordered(data):
 
     In [3]: trimesh.grouping.unique_ordered(a)
     Out[3]: array([0, 3, 4, 1, 2])
-    '''
+    """
     data = np.asanyarray(data)
     order = np.sort(np.unique(data, return_index=True)[1])
     result = data[order]
@@ -156,8 +190,10 @@ def unique_ordered(data):
 
 
 def merge_runs(data, digits=None):
-    '''
-    Merge duplicate sequential values.
+    """
+    Merge duplicate sequential values. This differs from unique_ordered
+    in that values can occur in multiple places in the sequence, but
+    only consecutive repeats are removed
 
     Parameters
     -----------
@@ -177,25 +213,24 @@ def merge_runs(data, digits=None):
 
     In [2]: trimesh.grouping.merge_runs(a)
     Out[2]: array([-1,  0,  1,  2,  0,  3,  4,  5,  6,  7,  8,  9])
-    '''
+    """
     data = np.asanyarray(data)
-    data_int = float_to_int(data, digits=digits)
-    mask = np.append(True, np.diff(data_int).astype(bool))
+    mask = np.abs(np.diff(data)) > tol.merge
+    mask = np.concatenate((np.array([True]), mask))
 
-    result = data[mask]
-    return result
+    return data[mask]
 
 
 def unique_float(data,
                  return_index=False,
                  return_inverse=False,
                  digits=None):
-    '''
+    """
     Identical to the numpy.unique command, except evaluates floating point
     numbers, using a specified number of digits.
 
     If digits isn't specified, the libray default TOL_MERGE will be used.
-    '''
+    """
     data = np.asanyarray(data)
     as_int = float_to_int(data, digits)
     _junk, unique, inverse = np.unique(as_int,
@@ -215,7 +250,7 @@ def unique_float(data,
 
 
 def unique_rows(data, digits=None):
-    '''
+    """
     Returns indices of unique rows. It will return the
     first occurrence of a row that is duplicated:
     [[1,2], [3,4], [1,2]] will return [0,1]
@@ -230,7 +265,7 @@ def unique_rows(data, digits=None):
     unique:  (j) array, index in data which is a unique row
     inverse: (n) length array to reconstruct original
                  example: unique[inverse] == data
-    '''
+    """
     hashes = hashable_rows(data, digits=digits)
     garbage, unique, inverse = np.unique(hashes,
                                          return_index=True,
@@ -239,7 +274,7 @@ def unique_rows(data, digits=None):
 
 
 def unique_value_in_row(data, unique=None):
-    '''
+    """
     For a 2D array of integers find the position of a value in each
     row which only occurs once. If there are more than one value per
     row which occur once, the last one is returned.
@@ -276,7 +311,7 @@ def unique_value_in_row(data, unique=None):
 
     In [3]: r[unique_value_in_row(r)]
     Out[3]: array([-1,  1, -1,  1,  1], dtype=int8)
-    '''
+    """
     if unique is None:
         unique = np.unique(data)
     data = np.asanyarray(data)
@@ -289,7 +324,7 @@ def unique_value_in_row(data, unique=None):
 
 
 def group_rows(data, require_count=None, digits=None):
-    '''
+    """
     Returns index groups of duplicate rows, for example:
     [[1,2], [3,4], [1,2]] will return [[0,2], [1]]
 
@@ -313,14 +348,14 @@ def group_rows(data, require_count=None, digits=None):
     groups:        List or sequence of indices from data indicating identical rows.
                    If require_count != None, shape will be (j, require_count)
                    If require_count is None, shape will be irregular (AKA a sequence)
-    '''
+    """
 
     def group_dict():
-        '''
+        """
         Simple hash table based grouping.
         The loop and appends make this rather slow on very large arrays,
         but it works on irregular groups.
-        '''
+        """
         observed = dict()
         hashable = hashable_rows(data, digits=digits)
         for index, key in enumerate(hashable):
@@ -349,7 +384,7 @@ def group_rows(data, require_count=None, digits=None):
         # you could use: np.array_split(dupe_idx)
         # this is roughly 3x slower than using the group_dict method above.
         start_ok = np.diff(
-            np.hstack((dupe_idx, len(hashable)))) == require_count
+            np.concatenate((dupe_idx, [len(hashable)]))) == require_count
         groups = np.tile(dupe_idx[start_ok].reshape((-1, 1)),
                          require_count) + np.arange(require_count)
         groups_idx = order[groups]
@@ -363,8 +398,8 @@ def group_rows(data, require_count=None, digits=None):
         return group_slice()
 
 
-def boolean_rows(a, b, operation=set.intersection):
-    '''
+def boolean_rows(a, b, operation=np.intersect1d):
+    """
     Find the rows in two arrays which occur in both rows.
 
     Parameters
@@ -379,18 +414,21 @@ def boolean_rows(a, b, operation=set.intersection):
     Returns
     --------
     shared: (p, d) array containing rows in both a and b
-    '''
-    a = float_to_int(a)
-    b = float_to_int(b)
-    shared = operation({tuple(i) for i in a}, {tuple(j) for j in b})
-    shared = np.array(list(shared))
+    """
+    a = np.asanyarray(a)
+    b = np.asanyarray(b)
+
+    av = a.view([('', a.dtype)] * a.shape[1]).ravel()
+    bv = b.view([('', b.dtype)] * b.shape[1]).ravel()
+    shared = operation(av, bv).view(a.dtype).reshape(-1, a.shape[1])
+
     return shared
 
 
 def group_vectors(vectors,
                   angle=1e-4,
                   include_negative=False):
-    '''
+    """
     Group vectors based on an angle tolerance, with the option to
     include negative vectors.
 
@@ -405,7 +443,7 @@ def group_vectors(vectors,
     ------------
     new_vectors: (m,3) float, vectors in space
     groups:      (m,) sequence of indicies in source vectors
-    '''
+    """
 
     vectors = np.asanyarray(vectors, dtype=np.float64)
     angle = float(angle)
@@ -420,18 +458,38 @@ def group_vectors(vectors,
 
 
 def group_distance(values, distance):
-    consumed = np.zeros(len(values), dtype=np.bool)
+    """
+    Find groups of points which have neighbours closer than radius,
+    where no two points in a group are farther than distance apart.
+
+    Parameters
+    ---------
+    points:   (n, d) points (of dimension d)
+    distance: max distance between points in a cluster
+
+    Returns
+    ----------
+    unique: (m, d), median value of group
+    groups: (m)     sequence of indexes
+
+    """
+    values = np.asanyarray(values,
+                           dtype=np.float64)
+
+    consumed = np.zeros(len(values),
+                        dtype=np.bool)
     tree = KDTree(values)
 
     # (n, d) set of values that are unique
-    unique = deque()
+    unique = []
     # (n) sequence of indicies in values
-    groups = deque()
+    groups = []
 
     for index, value in enumerate(values):
         if consumed[index]:
             continue
-        group = np.array(tree.query_ball_point(value, distance), dtype=np.int)
+        group = np.array(tree.query_ball_point(value, distance),
+                         dtype=np.int)
         consumed[group] = True
         unique.append(np.median(values[group], axis=0))
         groups.append(group)
@@ -439,7 +497,7 @@ def group_distance(values, distance):
 
 
 def clusters(points, radius):
-    '''
+    """
     Find clusters of points which have neighbours closer than radius
 
     Parameters
@@ -451,18 +509,19 @@ def clusters(points, radius):
     ----------
     groups: (m) sequence of indices for points
 
-    '''
+    """
     from . import graph
-
     tree = KDTree(points)
-    pairs = tree.query_pairs(radius)
+
+    # some versions return pairs as a set of tuples
+    pairs = list(tree.query_pairs(radius))
     groups = graph.connected_components(pairs)
 
     return groups
 
 
 def blocks(data, min_len=2, max_len=np.inf, digits=None, only_nonzero=False):
-    '''
+    """
     Given an array, find the indices of contiguous blocks
     of equal values.
 
@@ -477,14 +536,14 @@ def blocks(data, min_len=2, max_len=np.inf, digits=None, only_nonzero=False):
     Returns
     ---------
     blocks: (m) sequence of indices referencing data
-    '''
+    """
     data = float_to_int(data, digits=digits)
 
     # find the inflection points, or locations where the array turns
     # from True to False.
-    infl = np.hstack(([0],
-                      np.nonzero(np.diff(data))[0] + 1,
-                      [len(data)]))
+    infl = np.concatenate(([0],
+                           np.nonzero(np.diff(data))[0] + 1,
+                           [len(data)]))
     infl_len = np.diff(infl)
     infl_ok = np.logical_and(infl_len >= min_len,
                              infl_len <= max_len)
@@ -502,7 +561,7 @@ def blocks(data, min_len=2, max_len=np.inf, digits=None, only_nonzero=False):
 
 
 def merge_intervals(intervals):
-    '''
+    """
     Given a list of intervals, merge overlapping ranges into a single list
     of non- overlapping ranges
 
@@ -513,7 +572,7 @@ def merge_intervals(intervals):
     Returns
     -----------
     merged: (m,2) list of [start, end] values of ranges with no overlaps
-    '''
+    """
     def merge_generator(intervals):
         intervals.sort(axis=1)
         intervals = intervals[np.lexsort(intervals.T[::-1])]
