@@ -6,7 +6,6 @@ from string import Template
 import collections
 import subprocess
 import tempfile
-import base64
 import json
 
 from .. import util
@@ -19,9 +18,11 @@ ply_dtypes = {'char': 'i1',
               'short': 'i2',
               'ushort': 'u2',
               'int': 'i4',
+              'int8': 'i1',
               'int16': 'i2',
               'int32': 'i4',
               'uint': 'u4',
+              'uint8': 'u1',
               'uint16': 'u2',
               'uint32': 'u4',
               'float': 'f4',
@@ -157,11 +158,13 @@ def read_ply_header(file_obj):
                     str(field)] = endian + ply_dtypes[dtype]
             elif 'list' in line[1]:
                 dtype_count, dtype, field = line[2:]
-                elements[name]['properties'][str(field)] = (endian +
-                                                            ply_dtypes[dtype_count] +
-                                                            ', ($LIST,)' +
-                                                            endian +
-                                                            ply_dtypes[dtype])
+                elements[name]['properties'][
+                    str(field)] = (
+                    endian +
+                    ply_dtypes[dtype_count] +
+                    ', ($LIST,)' +
+                    endian +
+                    ply_dtypes[dtype])
     return elements, encoding_ascii
 
 
@@ -355,7 +358,7 @@ def ply_binary(elements, file_obj):
                     else:
                         offset = np.dtype(prior_data).itemsize
                     file_obj.seek(p_current + offset)
-                    size = np.fromstring(file_obj.read(field_dtype.itemsize),
+                    size = np.frombuffer(file_obj.read(field_dtype.itemsize),
                                          dtype=field_dtype)[0]
                     props[k] = props[k].replace('$LIST', str(size))
                 prior_data += props[k] + ','
@@ -372,7 +375,8 @@ def ply_binary(elements, file_obj):
             items = list(elements[key]['properties'].items())
             dtype = np.dtype(items)
             data = file_obj.read(elements[key]['length'] * dtype.itemsize)
-            elements[key]['data'] = np.fromstring(data, dtype=dtype)
+            elements[key]['data'] = np.frombuffer(data,
+                                                  dtype=dtype)
         return elements
 
     def elements_size(elements):
