@@ -68,9 +68,21 @@ def load_stl_binary(file_obj):
     header_length = _stl_dtype_header.itemsize
     header_data = file_obj.read(header_length)
     if len(header_data) < header_length:
-        raise HeaderError('Binary STL file not long enough to contain header!')
+        raise HeaderError('Binary STL shorter than a fixed header!')
 
-    header = np.fromstring(header_data, dtype=_stl_dtype_header)
+    try:
+        header = np.frombuffer(header_data,
+                               dtype=_stl_dtype_header)
+    except BaseException:
+        raise HeaderError('Binary header incorrect type')
+
+    try:
+        # save the header block as a string
+        # there could be any garbage in there so wrap in try
+        metadata = {'header':
+                    bytes(header['header'][0]).decode('utf-8').strip()}
+    except BaseException:
+        metadata = {}
 
     # now we check the length from the header versus the length of the file
     # data_start should always be position 84, but hard coding that felt ugly
@@ -95,15 +107,16 @@ def load_stl_binary(file_obj):
     # so it's much better to raise an exception here.
     if len_data != len_expected:
         raise HeaderError('Binary STL has incorrect length in header!')
+    blob = np.frombuffer(file_obj.read(), dtype=_stl_dtype)
 
-    # all of our vertices will be loaded in order due to the STL format,
+    # all of our vertices will be loaded in order
     # so faces are just sequential indices reshaped.
     faces = np.arange(header['face_count'] * 3).reshape((-1, 3))
-    blob = np.fromstring(file_obj.read(), dtype=_stl_dtype)
 
     result = {'vertices': blob['vertices'].reshape((-1, 3)),
               'face_normals': blob['normals'].reshape((-1, 3)),
-              'faces': faces}
+              'faces': faces,
+              'metadata': metadata}
     return result
 
 
@@ -140,11 +153,11 @@ def load_stl_ascii(file_obj):
     # this offset is to be added to a fixed set of indices that is tiled
     offset = face_len * np.arange(face_count).reshape((-1, 1))
     normal_index = np.tile([2, 3, 4], (face_count, 1)) + offset
-    vertex_index = np.tile(
-        [8, 9, 10, 12, 13, 14, 16, 17, 18], (face_count, 1)) + offset
+    vertex_index = np.tile([8, 9, 10,
+                            12, 13, 14,
+                            16, 17, 18], (face_count, 1)) + offset
 
-    # faces are groups of three sequential vertices, as vertices are not
-    # references
+    # faces are groups of three sequential vertices
     faces = np.arange(face_count * 3).reshape((-1, 3))
     face_normals = blob[normal_index].astype(np.float64)
     vertices = blob[vertex_index.reshape((-1, 3))].astype(np.float64)
