@@ -2,7 +2,6 @@ import logging
 import os
 import subprocess
 
-import lxml.etree as et
 import numpy as np
 
 from ..decomposition import convex_decomposition
@@ -10,12 +9,15 @@ from .. import io
 from ..version import __version__ as trimesh_version
 
 
-def export_urdf(mesh, directory, scale=1.0, color=[
-                0.75, 0.75, 0.75], **kwargs):
+def export_urdf(mesh,
+                directory,
+                scale=1.0,
+                color=[0.75, 0.75, 0.75],
+                **kwargs):
     '''
     Convert a Trimesh object into a URDF package for physics simulation.
     This breaks the mesh into convex pieces and writes them to the same
-    directory as the .urdf file. For added convenience, this also
+    directory as the .urdf file.
 
     Parameters
     ---------
@@ -27,6 +29,8 @@ def export_urdf(mesh, directory, scale=1.0, color=[
     mesh: The decomposed mesh
     '''
 
+    import lxml.etree as et
+
     # Extract the save directory and the file name
     fullpath = os.path.abspath(directory)
     name = os.path.basename(fullpath)
@@ -80,14 +84,21 @@ def export_urdf(mesh, directory, scale=1.0, color=[
         inertial = et.SubElement(link, 'inertial')
         et.SubElement(inertial, 'origin', xyz="0 0 0", rpy="0 0 0")
         et.SubElement(inertial, 'mass', value='{:.2E}'.format(piece.mass))
-        et.SubElement(inertial, 'inertia', ixx=I[0][0], ixy=I[0][1], ixz=I[0][2],
-                      iyy=I[1][1], iyz=I[1][2], izz=I[2][2])
+        et.SubElement(
+            inertial,
+            'inertia',
+            ixx=I[0][0],
+            ixy=I[0][1],
+            ixz=I[0][2],
+            iyy=I[1][1],
+            iyz=I[1][2],
+            izz=I[2][2])
         # Visual Information
         visual = et.SubElement(link, 'visual')
         et.SubElement(visual, 'origin', xyz="0 0 0", rpy="0 0 0")
         geometry = et.SubElement(visual, 'geometry')
         et.SubElement(geometry, 'mesh', filename=geom_name,
-                      scale="{:.4E}".format(scale))
+                      scale="{:.4E} {:.4E} {:.4E}".format(scale, scale, scale))
         material = et.SubElement(visual, 'material', name='')
         et.SubElement(material, 'color', rgba="{:.2E} {:.2E} {:.2E} 1".format(
             color[0], color[1], color[2]))
@@ -97,7 +108,7 @@ def export_urdf(mesh, directory, scale=1.0, color=[
         et.SubElement(collision, 'origin', xyz="0 0 0", rpy="0 0 0")
         geometry = et.SubElement(collision, 'geometry')
         et.SubElement(geometry, 'mesh', filename=geom_name,
-                      scale="{:.4E}".format(scale))
+                      scale="{:.4E} {:.4E} {:.4E}".format(scale, scale, scale))
 
         # Create rigid joint to previous link
         if prev_link_name is not None:
