@@ -8,10 +8,10 @@ from .intersections import line_line
 
 
 def arc_center(points):
-    '''
+    """
     Given three points of an arc, find the center, radius, normal, and angle.
 
-    This uses the fact that the intersection of the perpendicular
+    This uses the fact that the intersection of the perp
     bisectors of the segments between the control points is the center of the arc.
 
     Parameters
@@ -26,7 +26,7 @@ def arc_center(points):
         'normal':   (3,) float, the plane normal.
         'angle':    (2,) float, angle of start and end, in radians
         'span' :    float, angle swept by the arc, in radians
-    '''
+    """
     # it's a lot easier to treat 2D as 3D with a zero Z value
     is_2D, points = three_dimensionalize(points, return_2D=True)
 
@@ -35,17 +35,32 @@ def arc_center(points):
     edge_midpoints = (edge_direction * .5) + points[0:2]
 
     # three points define a plane, so we find its normal vector
-    plane_normal = unitize(np.cross(*edge_direction[::-1]))
-    vector_edge = unitize(edge_direction)
-    vector_perpendicular = unitize(np.cross(vector_edge, plane_normal))
+    plane_normal = np.cross(*edge_direction[::-1])
+    plane_normal /= np.linalg.norm(plane_normal)
 
-    intersects, center = line_line(edge_midpoints, vector_perpendicular)
+    # unit vector along edges
+    vector_edge = (edge_direction /
+                   np.linalg.norm(edge_direction, axis=1).reshape((-1, 1)))
+
+    # perpendicular cector to each segment
+    vector_perp = np.cross(vector_edge, plane_normal)
+    vector_perp /= np.linalg.norm(vector_perp, axis=1).reshape((-1, 1))
+
+    # run the line- line intersection to find the point
+    intersects, center = line_line(origins=edge_midpoints,
+                                   directions=vector_perp,
+                                   plane_normal=plane_normal)
 
     if not intersects:
         raise ValueError('Segments do not intersect!')
 
-    radius = euclidean(points[0], center)
-    vector = unitize(points - center)
+    # radius is euclidean distance
+    radius = ((points[0] - center) ** 2).sum() ** .5
+
+    # vectors from points on arc to center point
+    vector = points - center
+    vector /= np.linalg.norm(vector, axis=1).reshape((-1, 1))
+
     angle = np.arccos(np.clip(np.dot(*vector[[0, 2]]), -1.0, 1.0))
     large_arc = (abs(angle) > tol.zero and
                  np.dot(*edge_direction) < 0.0)
@@ -66,7 +81,7 @@ def arc_center(points):
 
 
 def discretize_arc(points, close=False, scale=1.0):
-    '''
+    """
     Returns a version of a three point arc consisting of line segments
 
     Parameters
@@ -79,7 +94,7 @@ def discretize_arc(points, close=False, scale=1.0):
     discrete: (m, d)
     points: either (3,3) or (3,2) of points for arc going from
             points[0] to points[2], going through control point points[1]
-    '''
+    """
     two_dimensional, points = three_dimensionalize(points, return_2D=True)
     center_info = arc_center(points)
     center, R, N, angle = (center_info['center'],
@@ -121,9 +136,9 @@ def discretize_arc(points, close=False, scale=1.0):
 
 
 def arc_tangents(points):
-    '''
+    """
     returns tangent vectors for points
-    '''
+    """
     two_dimensional, points = three_dimensionalize(points, return_2D=True)
     center, R, N, angle = arc_center(points)
     vectors = points - center
@@ -139,10 +154,46 @@ def arc_offset(points, distance):
     return new_points[:, 0:(3 - two_dimensional)]
 
 
-def angles_to_threepoint(angles, center, radius):
+def to_threepoint(center, radius, angles=None):
+    """
+    For 2D arcs, given a center and radius convert them to three
+    points on the arc.
+
+    Parameters
+    -----------
+    center: (2,) float, center point on the plane
+    radius: float, radius of arc
+    angles: (2,) float, angles in radians to make the arc
+                 if not specified, will default to (0.0, pi)
+
+    Returns
+    ----------
+    three: (3,2) float, arc control points
+    """
+    # if no angles provided assume we want a half circle
+    if angles is None:
+        angles = [0.0, np.pi]
+    # force angles to float64
+    angles = np.asanyarray(angles, dtype=np.float64)
+    if angles.shape != (2,):
+        raise ValueError('angles must be (2,)!')
+    # provide the wrap around
     if angles[1] < angles[0]:
         angles[1] += np.pi * 2
-    angles = np.array([angles[0], np.mean(angles),
-                       angles[1]], dtype=np.float64)
-    planar = np.column_stack((np.cos(angles), np.sin(angles))) * radius
-    return planar + center
+
+    center = np.asanyarray(center, dtype=np.float64)
+    if center.shape != (2,):
+        raise ValueError('only valid on 2D arcs!')
+
+    # turn the angles of [start, end]
+    # into [start, middle, end]
+    angles = np.array([angles[0],
+                       angles.mean(),
+                       angles[1]],
+                      dtype=np.float64)
+    # turn angles into (3,2) points
+    three = np.column_stack((np.cos(angles),
+                             np.sin(angles))) * radius
+    three += center
+
+    return three
