@@ -5,19 +5,19 @@ from shapely.geometry import Polygon, Point, LineString
 from rtree import Rtree
 from collections import deque
 
+from .. import util
 from .. import bounds
 from .. import graph
 
-from ..geometry import medial_axis as _medial_axis
 from ..constants import tol_path as tol
 from ..constants import log
-from ..transformations import transform_points, planar_matrix
-from ..util import is_sequence
+from ..transformations import transform_points
+
 from .traversal import resample_path
 
 
 def enclosure_tree(polygons):
-    '''
+    """
     Given a list of shapely polygons, which are the root (aka outermost)
     polygons, and which represent the holes which penetrate the root
     curve. We do this by creating an R-tree for rough collision detection,
@@ -32,12 +32,14 @@ def enclosure_tree(polygons):
     roots: (m,) int, index of polygons which are root
     contains:  networkx.DiGraph, edges indicate a polygon
                contained by another polygon
-    '''
+    """
     tree = Rtree()
     for i, polygon in enumerate(polygons):
         tree.insert(i, polygon.bounds)
     count = len(polygons)
+    # nodes are indexes in polygons
     contains = nx.DiGraph()
+    # make sure every polygon is included
     contains.add_nodes_from(np.arange(count))
     for i in range(count):
         if polygons[i] is None:
@@ -57,7 +59,7 @@ def enclosure_tree(polygons):
 
 
 def edges_to_polygons(edges, vertices):
-    '''
+    """
     Given an edge list of indices and associated vertices
     representing lines, generate a list of polygons.
 
@@ -69,11 +71,17 @@ def edges_to_polygons(edges, vertices):
     Returns
     ----------
     polygons: (p,) list of shapely.geometry.Polygon objects
-    '''
-    # sequence of ordered traversals
-    dfs = graph.dfs_traversals(edges)
+    """
+
     # create closed polygon objects
-    polygons = [Polygon(vertices[i]) for i in dfs]
+    polygons = []
+    # loop through a sequence of ordered traversals
+    for dfs in graph.dfs_traversals(edges):
+        try:
+            # try to recover polygons before they are more complicated
+            polygons.append(repair_invalid(Polygon(vertices[dfs])))
+        except ValueError:
+            continue
 
     # if there is only one polygon, just return it
     if len(polygons) == 1:
@@ -94,9 +102,9 @@ def edges_to_polygons(edges, vertices):
 
 
 def polygons_obb(polygons):
-    '''
+    """
     Find the OBBs for a list of shapely.geometry.Polygons
-    '''
+    """
     rectangles = [None] * len(polygons)
     transforms = [None] * len(polygons)
     for i, p in enumerate(polygons):
@@ -105,7 +113,7 @@ def polygons_obb(polygons):
 
 
 def polygon_obb(polygon):
-    '''
+    """
     Find the oriented bounding box of a Shapely polygon.
 
     The OBB is always aligned with an edge of the convex hull of the polygon.
@@ -120,13 +128,18 @@ def polygon_obb(polygon):
                which will move input polygon from its original position
                to the first quadrant where the AABB is the OBB
     extents:   (2,) float, extents of transformed polygon
-    '''
-    points = np.asanyarray(polygon.exterior.coords)
+    """
+    if hasattr(polygon, 'exterior'):
+        points = np.asanyarray(polygon.exterior.coords)
+    elif isinstance(polygon, np.ndarray):
+        points = polygon
+    else:
+        raise ValueError('polygon or points must be provided')
     return bounds.oriented_bounds_2D(points)
 
 
 def transform_polygon(polygon, transform, plot=False):
-    if is_sequence(polygon):
+    if util.is_sequence(polygon):
         result = [transform_polygon(p, t) for p, t in zip(polygon, transform)]
     else:
         shell = transform_points(np.array(polygon.exterior.coords), transform)
@@ -138,103 +151,48 @@ def transform_polygon(polygon, transform, plot=False):
     return result
 
 
-def rasterize_polygon(polygon, pitch):
-    '''
-    Given a shapely polygon, find the raster representation at a given angle
-    relative to the oriented bounding box
+def plot_polygon(polygon, show=True):
+    """
+    Plot a shapely polygon using matplotlib.
 
     Parameters
-    ----------
-    polygon: shapely polygon
-    pitch:   what is the edge length of a pixel
-
-    Returns
-    ----------
-    offset:      (2,) float, where the origin of the raster array is located
-    grid:        (n,m) bool, where filled areas are True
-    grid_points: (p,2) float, points in space
-    '''
-
-    bounds = np.reshape(polygon.bounds, (2, 2))
-    offset = bounds[0]
-    shape = np.ceil(np.ptp(bounds, axis=0) / pitch).astype(int)
-    grid = np.zeros(shape, dtype=np.bool)
-
-    def fill(ranges):
-        ranges = (np.array(ranges) - offset[0]) / pitch
-        x_index = np.array([np.floor(ranges[0]),
-                            np.ceil(ranges[1])]).astype(int)
-        if np.any(x_index < 0):
-            return
-        grid[x_index[0]:x_index[1], y_index] = True
-        if (y_index > 0):
-            grid[x_index[0]:x_index[1], y_index - 1] = True
-
-    def handler_multi(geometries):
-        for geometry in geometries:
-            handlers[geometry.__class__.__name__](geometry)
-
-    def handler_line(line):
-        fill(line.xy[0])
-
-    def handler_null(data):
-        pass
-
-    handlers = {'GeometryCollection': handler_multi,
-                'MultiLineString': handler_multi,
-                'MultiPoint': handler_multi,
-                'LineString': handler_line,
-                'Point': handler_null}
-
-    x_extents = bounds[:, 0] + [-pitch, pitch]
-    for y_index in range(grid.shape[1]):
-        y = offset[1] + y_index * pitch
-        test = LineString(np.column_stack((x_extents, [y, y])))
-        hits = polygon.intersection(test)
-        handlers[hits.__class__.__name__](hits)
-
-    grid_points = ((np.transpose(np.nonzero(grid)).astype(
-        np.float64) * pitch) + offset + (pitch / 2.0))
-
-    return offset, grid, grid_points
-
-
-def plot_polygon(polygon, show=True):
+    ------------
+    polygon: shapely.geometry.Polygon object
+    show:    bool, if True will display immediatly
+    """
     import matplotlib.pyplot as plt
 
     def plot_single(single):
         plt.plot(*single.exterior.xy, color='b')
         for interior in single.interiors:
             plt.plot(*interior.xy, color='r')
-
+    # make aspect ratio non- stupid
     plt.axes().set_aspect('equal', 'datalim')
-    if is_sequence(polygon):
+    if util.is_sequence(polygon):
         [plot_single(i) for i in polygon]
     else:
         plot_single(polygon)
+
     if show:
         plt.show()
 
 
-def plot_raster(raster, pitch, offset=[0, 0]):
-    '''
-    Plot a raster representation.
-
-    raster: (n,m) array of booleans, representing filled/empty area
-    pitch:  the edge length of a box from raster, in cartesian space
-    offset: offset in cartesian space to the lower left corner of the raster grid
-    '''
-    import matplotlib.pyplot as plt
-    plt.axes().set_aspect('equal', 'datalim')
-    filled = (np.column_stack(np.nonzero(raster)) * pitch) + offset
-    for location in filled:
-        plt.gca().add_patch(plt.Rectangle(location,
-                                          pitch,
-                                          pitch,
-                                          facecolor="grey"))
-
-
 def resample_boundaries(polygon, resolution, clip=None):
+    """
+    Return a version of a polygon with boundaries resampled
+    to a specified resolution.
+
+    Parameters
+    -------------
+    polygon:    shapely.geometry.Polygon object
+    resolution: float, desired distance between points on boundary
+    clip:       (2,) int, upper and lower bounds to clip
+                number of samples to (to avoid exploding counts)
+
+    Returns
+    ------------
+    kwargs: dict, keyword args for a Polygon(**kwargs)
+    """
     def resample_boundary(boundary):
         # add a polygon.exterior or polygon.interior to
         # the deque after resampling based on our resolution
@@ -244,15 +202,27 @@ def resample_boundaries(polygon, resolution, clip=None):
     if clip is None:
         clip = [8, 200]
     # create a sequence of [(n,2)] points
-    result = {'shell': resample_boundary(polygon.exterior),
+    kwargs = {'shell': resample_boundary(polygon.exterior),
               'holes': deque()}
     for interior in polygon.interiors:
-        result['holes'].append(resample_boundary(interior))
-    result['holes'] = np.array(result['holes'])
-    return result
+        kwargs['holes'].append(resample_boundary(interior))
+    kwargs['holes'] = np.array(kwargs['holes'])
+    return kwargs
 
 
 def stack_boundaries(boundaries):
+    """
+    Stack the boundaries of a polygon into a single
+    (n, 2) list of vertices.
+
+    Parameters
+    ------------
+    boundaries: dict, with keys 'shell', 'holes'
+
+    Returns
+    ------------
+    stacked: (n, 2) float, list of vertices
+    """
     if len(boundaries['holes']) == 0:
         return boundaries['shell']
     result = np.vstack((boundaries['shell'],
@@ -261,7 +231,7 @@ def stack_boundaries(boundaries):
 
 
 def medial_axis(polygon, resolution=.01, clip=None):
-    '''
+    """
     Given a shapely polygon, find the approximate medial axis based
     on a voronoi diagram of evenly spaced points on the boundary of the polygon.
 
@@ -278,26 +248,46 @@ def medial_axis(polygon, resolution=.01, clip=None):
     Returns
     ----------
     lines:     (n,2,2) set of line segments
-    '''
-    def contains(points):
-        return np.array([polygon.contains(Point(i)) for i in points])
+    """
+    from scipy.spatial import Voronoi
+    from .entities import Line
+    from .path import Path2D
 
-    boundary = resample_boundaries(polygon=polygon,
-                                   resolution=resolution,
-                                   clip=clip)
-    boundary = stack_boundaries(boundary)
+    samples = resample_boundaries(polygon=polygon,
+                                  resolution=resolution,
+                                  clip=clip)
+    samples = stack_boundaries(samples)
 
-    return _medial_axis(samples=boundary,
-                        contains=contains)
+    # create the voronoi diagram, after vertically stacking the points
+    # deque from a sequnce into a clean (m,2) array
+    voronoi = Voronoi(samples)
+    # which voronoi vertices are contained inside the original polygon
+    contain = np.array([polygon.contains(Point(i))
+                        for i in voronoi.vertices],
+                       dtype=np.bool)
+    # ridge vertices of -1 are outside, make sure they are False
+    contain = np.append(contain, False)
+    # is a ridge fully inside the polygon
+    inside = [i for i in voronoi.ridge_vertices if contain[i].all()]
+    edges = np.vstack([util.stack_lines(i)
+                       for i in inside if len(i) >= 2])
+    # line objects from edges
+    entities = [Line(points=i) for i in edges]
+
+    # create the Path2D object for the medial axis
+    medial = Path2D(entities=entities,
+                    vertices=voronoi.vertices)
+
+    return medial
 
 
 class InversePolygon:
-    '''
+    """
     Create an inverse polygon.
 
     The primary use case is that given a point inside a polygon,
     you want to find the minimum distance to the boundary of the polygon.
-    '''
+    """
 
     def __init__(self, polygon):
         _DIST_BUFFER = .05
@@ -320,7 +310,7 @@ class InversePolygon:
         self._polygons = np.append(exterior, interiors)
 
     def distances(self, point):
-        '''
+        """
         Find the minimum distances from a point to the exterior and interiors
 
         Parameters
@@ -330,12 +320,12 @@ class InversePolygon:
         Returns
         ---------
         distances: (n) list of floats
-        '''
+        """
         distances = [i.distance(Point(point)) for i in self._polygons]
         return distances
 
     def distance(self, point):
-        '''
+        """
         Find the minimum distance from a point to the boundary of the polygon.
 
         Parameters
@@ -345,13 +335,13 @@ class InversePolygon:
         Returns
         ---------
         distance: float
-        '''
+        """
         distance = np.min(self.distances(point))
         return distance
 
 
 def polygon_hash(polygon):
-    '''
+    """
     An approximate hash of a a shapely Polygon object.
 
     Parameters
@@ -361,7 +351,7 @@ def polygon_hash(polygon):
     Returns
     ---------
     hash: (5) length list of hash representing input polygon
-    '''
+    """
     result = [len(polygon.interiors),
               polygon.convex_hull.area,
               polygon.convex_hull.length,
@@ -371,7 +361,7 @@ def polygon_hash(polygon):
 
 
 def random_polygon(segments=8, radius=1.0):
-    '''
+    """
     Generate a random polygon with a maximum number of sides and approximate radius.
 
     Parameters
@@ -382,7 +372,7 @@ def random_polygon(segments=8, radius=1.0):
     Returns
     ---------
     polygon: shapely.geometry.Polygon object with random exterior, and no interiors.
-    '''
+    """
     angles = np.sort(np.cumsum(np.random.random(
         segments) * np.pi * 2) % (np.pi * 2))
     radii = np.random.random(segments) * radius
@@ -390,19 +380,31 @@ def random_polygon(segments=8, radius=1.0):
         (np.cos(angles), np.sin(angles))) * radii.reshape((-1, 1))
     points = np.vstack((points, points[0]))
     polygon = Polygon(points).buffer(0.0)
-    if is_sequence(polygon):
+    if util.is_sequence(polygon):
         return polygon[0]
     return polygon
 
 
 def polygon_scale(polygon):
-    box = np.abs(np.diff(np.reshape(polygon, (2, 2)), axis=0))
-    scale = box.max()
+    """
+    For a Polygon object, return the diagonal length of the AABB.
+
+    Parameters
+    ------------
+    polygon: shapely.geometry.Polygon object
+
+    Returns
+    ------------
+    scale: float, length of AABB diagonal
+    """
+    extents = np.reshape(polygon.bounds, (2, 2)).ptp(axis=0)
+    scale = (extents ** 2).sum() ** .5
+
     return scale
 
 
 def paths_to_polygons(paths, scale=None):
-    '''
+    """
     Given a sequence of connected points turn them into
     valid shapely Polygon objects.
 
@@ -416,7 +418,7 @@ def paths_to_polygons(paths, scale=None):
     polys: (p,) list of shapely.geometry.Polygons
     valid: (n,) bool, whether input path was valid:
                         valid.sum() == p
-    '''
+    """
     polygons = []
     valid = np.zeros(len(paths), dtype=np.bool)
     for i, path in enumerate(paths):
@@ -435,14 +437,15 @@ def paths_to_polygons(paths, scale=None):
     return polygons, valid
 
 
-def repair_invalid(polygon, scale=None):
-    '''
+def repair_invalid(polygon, scale=None, rtol=.5):
+    """
     Given a shapely.geometry.Polygon, attempt to return a
-    valid version of the polygon.
+    valid version of the polygon through buffering tricks.
 
     Parameters
     -----------
     polygon: shapely.geometry.Polygon object
+    rtol:    float, how close does a perimeter have to be
     scale:   float, or None
 
     Returns
@@ -452,28 +455,53 @@ def repair_invalid(polygon, scale=None):
     Raises
     ----------
     ValueError: if polygon can't be repaired
-    '''
+    """
     if hasattr(polygon, 'is_valid') and polygon.is_valid:
         return polygon
 
     # basic repair involves buffering the polygon outwards
     # this will fix a subset of problems.
     basic = polygon.buffer(tol.zero)
+    # if it returned multiple polygons check the largest
+    if util.is_sequence(basic):
+        basic = basic[np.argmax([i.area for i in basic])]
 
-    if basic.area < tol.zero:
-        raise ValueError('zero area polygon')
-
-    if basic.is_valid:
+    # check perimeter of result agains original perimeter
+    if basic.is_valid and np.isclose(basic.length,
+                                     polygon.length,
+                                     rtol=rtol):
         return basic
 
     if scale is None:
-        scale = polygon_scale(polygon)
+        distance = tol.buffer * polygon_scale(polygon)
+    else:
+        distance = tol.buffer * scale
 
-    buffered = basic.buffer(tol.buffer * scale)
-    unbuffered = buffered.buffer(-tol.buffer * scale)
+    # if there are no interiors, we can work with just the exterior
+    # ring, which is often more reliable
+    if len(polygon.interiors) == 0:
+        # try buffering the exterior of the polygon
+        # the interior will be offset by -tol.buffer
+        rings = polygon.exterior.buffer(distance).interiors
+        if len(rings) == 1:
+            # reconstruct a single polygon from the interior ring
+            recon = Polygon(shell=rings[0]).buffer(distance)
+            # check perimeter of result agains original perimeter
+            if recon.is_valid and np.isclose(recon.length,
+                                             polygon.length,
+                                             rtol=rtol):
+                return recon
 
-    if unbuffered.is_valid and not is_sequence(unbuffered):
+    # buffer and unbuffer the whole polygon
+    buffered = polygon.buffer(distance).buffer(-distance)
+    # if it returned multiple polygons check the largest
+    if util.is_sequence(buffered):
+        buffered = buffered[np.argmax([i.area for i in buffered])]
+    # check perimeter of result agains original perimeter
+    if buffered.is_valid and np.isclose(buffered.length,
+                                        polygon.length,
+                                        rtol=rtol):
         log.debug('Recovered invalid polygon through double buffering')
-        return unbuffered
+        return buffered
 
-    raise ValueError('Unable to recover polygon!')
+    raise ValueError('unable to recover polygon!')
