@@ -1,6 +1,6 @@
-'''
-A basic, slow implementation of ray- triangle queries.
-'''
+"""
+A basic slow implementation of ray- triangle queries.
+"""
 import numpy as np
 
 
@@ -9,20 +9,21 @@ from .ray_util import contains_points
 from ..constants import tol
 
 from .. import util
+from .. import caching
 from .. import grouping
 from .. import intersections
 from .. import triangles as triangles_mod
 
 
 class RayMeshIntersector:
-    '''
+    """
     An object to query a mesh for ray intersections.
     Precomputes an r-tree for each triangle on the mesh.
-    '''
+    """
 
     def __init__(self, mesh):
         self.mesh = mesh
-        self._cache = util.Cache(self.mesh.crc)
+        self._cache = caching.Cache(self.mesh.crc)
 
     def intersects_id(self,
                       ray_origins,
@@ -30,7 +31,7 @@ class RayMeshIntersector:
                       return_locations=False,
                       multiple_hits=True,
                       **kwargs):
-        '''
+        """
         Find the intersections between the current mesh and a list of rays.
 
         Parameters
@@ -45,7 +46,7 @@ class RayMeshIntersector:
         index_triangle: (h,) int,    index of triangles hit
         index_ray:      (h,) int,    index of ray that hit triangle
         locations:      (h,3) float, (optional) position of intersection in space
-        '''
+        """
         (index_tri,
          index_ray,
          locations) = ray_triangle_id(triangles=self.mesh.triangles,
@@ -55,10 +56,10 @@ class RayMeshIntersector:
                                       multiple_hits=multiple_hits,
                                       triangles_normal=self.mesh.face_normals)
         if return_locations:
+            if len(index_tri) == 0:
+                return index_tri, index_ray, locations
             unique = grouping.unique_rows(np.column_stack((locations,
                                                            index_ray)))[0]
-            if len(unique) == 0:
-                return [], [], []
             return index_tri[unique], index_ray[unique], locations[unique]
         return index_tri, index_ray
 
@@ -66,7 +67,7 @@ class RayMeshIntersector:
                             ray_origins,
                             ray_directions,
                             **kwargs):
-        '''
+        """
         Return unique cartesian locations where rays hit the mesh.
         If you are counting the number of hits a ray had, this method
         should be used as if only the triangle index is used on- edge hits
@@ -82,7 +83,7 @@ class RayMeshIntersector:
         locations: (n) sequence of (m,3) intersection points
         index_ray: (n,) int, list of ray index
         index_tri: (n,) int, list of triangle (face) indexes
-        '''
+        """
         (index_tri,
          index_ray,
          locations) = self.intersects_id(ray_origins=ray_origins,
@@ -95,7 +96,7 @@ class RayMeshIntersector:
                        ray_origins,
                        ray_directions,
                        **kwargs):
-        '''
+        """
         Find out if each ray hit any triangle on the mesh.
 
         Parameters
@@ -106,7 +107,7 @@ class RayMeshIntersector:
         Returns
         ---------
         hit: boolean, whether any ray hit any triangle on the mesh
-        '''
+        """
         index_tri, index_ray = self.intersects_id(ray_origins,
                                                   ray_directions)
         hit_any = np.zeros(len(ray_origins), dtype=np.bool)
@@ -116,7 +117,7 @@ class RayMeshIntersector:
         return hit_any
 
     def contains_points(self, points):
-        '''
+        """
         Check if a mesh contains a list of points, using ray tests.
 
         If the point is on the surface of the mesh, behavior is undefined.
@@ -128,7 +129,7 @@ class RayMeshIntersector:
         Returns
         ---------
         contains: (n) boolean array, whether point is inside mesh or not
-        '''
+        """
 
         return contains_points(self, points)
 
@@ -139,7 +140,7 @@ def ray_triangle_id(triangles,
                     triangles_normal=None,
                     tree=None,
                     multiple_hits=True):
-    '''
+    """
     Find the intersections between a group of triangles and rays
 
     Parameters
@@ -155,7 +156,7 @@ def ray_triangle_id(triangles,
     index_triangle: (h,) int,    index of triangles hit
     index_ray:      (h,) int,    index of ray that hit triangle
     locations:      (h,3) float, position of intersection in space
-    '''
+    """
     triangles = np.asanyarray(triangles, dtype=np.float64)
     ray_origins = np.asanyarray(ray_origins, dtype=np.float64)
     ray_directions = np.asanyarray(ray_directions, dtype=np.float64)
@@ -198,8 +199,8 @@ def ray_triangle_id(triangles,
 
     # find the barycentric coordinates of each plane intersection on the
     # triangle candidates
-    barycentric = triangles_mod.points_to_barycentric(triangle_candidates[valid],
-                                                      location)
+    barycentric = triangles_mod.points_to_barycentric(
+        triangle_candidates[valid], location)
 
     # the plane intersection is inside the triangle if all barycentric coordinates
     # are between 0.0 and 1.0
@@ -245,7 +246,7 @@ def ray_triangle_id(triangles,
 def ray_triangle_candidates(ray_origins,
                             ray_directions,
                             tree):
-    '''
+    """
     Do broad- phase search for triangles that the rays
     may intersect.
 
@@ -262,7 +263,7 @@ def ray_triangle_candidates(ray_origins,
     ----------
     ray_candidates: (n,) int, triangle indexes
     ray_id:         (n,) int, corresponding ray index for a triangle candidate
-    '''
+    """
     ray_bounding = ray_bounds(ray_origins=ray_origins,
                               ray_directions=ray_directions,
                               bounds=tree.bounds)
@@ -284,7 +285,7 @@ def ray_bounds(ray_origins,
                ray_directions,
                bounds,
                buffer_dist=1e-5):
-    '''
+    """
     Given a set of rays and a bounding box for the volume of interest
     where the rays will be passing through, find the bounding boxes
     of the rays as they pass through the volume.
@@ -299,7 +300,7 @@ def ray_bounds(ray_origins,
     Returns
     ---------
     ray_bounding: (n) set of AABB of rays passing through volume
-    '''
+    """
 
     # bounding box we are testing against
     bounds = np.array(bounds)
